﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Configuration;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using J2.eFaxDeveloper.Outbound;
using J2.eFaxDeveloper.Inbound;
using J2.eFaxDeveloper.Sample.WebApplication.Models;

namespace J2.eFaxDeveloper.Sample.WebApplication.Controllers
{
    public class eFaxDeveloperController : Controller
    {
        /// <summary>
        /// Custom action filter that check if the OutboundClient is ready to use
        /// </summary>
        internal class CheckFaxSettingsAttribute : ActionFilterAttribute 
        {
            public override void OnActionExecuting(ActionExecutingContext filterContext) 
            {
                try
                {
                    OutboundClient outboundClient = GetOutboundClient();
                }
                catch (Exception)
                {
                    // redirect the user to the setting page
                    RequestContext rc = new RequestContext(filterContext.HttpContext, filterContext.RouteData);
                    string url = RouteTable.Routes.GetVirtualPath(rc, new RouteValueDictionary(new 
                        { Controller = "eFaxDeveloper", action = "Settings" })).VirtualPath;
                    filterContext.HttpContext.Response.Redirect(url, true); 
                }

                base.OnActionExecuting(filterContext);
            }
        }

        /// <summary>
        ///  Get an instance of OutboundClient and load its configuration
        /// </summary>
        /// <returns></returns>
        private static OutboundClient GetOutboundClient()
        {
            OutboundClient outboundClient = new OutboundClient();

            // this will throw a exception if the configuration is not found or corrupted
            outboundClient.LoadWebConfiguration();

            return outboundClient;
        }

        /// <summary>
        /// Settings for efaxDevelopper
        /// </summary>
        /// <returns></returns>
        [RedirectToHttps]
        public ActionResult Settings()
        {
            try
            {
                return View(GetOutboundClient());
            }
            catch
            {
            }

            return View(new OutboundClient());
        }

        /// <summary>
        /// Settings for efaxDevelopper
        /// </summary>
        /// <param name="outboundClient"></param>
        /// <returns></returns>
        [HttpPost]
        public ActionResult Settings(OutboundClient outboundClient)
        {
            try
            {
                // save and encrypt efaxDeveloper settings
                outboundClient.SaveWebConfiguration(true);

                return RedirectToAction("Index", "Home");
            }
            catch
            {
                return View();
            }
        }

        /// <summary>
        /// Send fax
        /// </summary>
        /// <returns></returns>
        [CheckFaxSettings]
        public ActionResult SendFax()
        {
            try
            {
                return View(new TestFax { DispositionEmail = Membership.GetUser().Email });
            }
            catch
            {
                return View();
            }
        }

        /// <summary>
        /// Send fax
        /// </summary>
        /// <param name="testFax"></param>
        /// <returns></returns>
        [HttpPost]
        public ActionResult SendFax(TestFax testFax)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    // create an instance of OutboundRequest
                    OutboundRequest outboundRequest = new OutboundRequest();

                    // set the fax recipient
                    outboundRequest.Recipient.RecipientFax = testFax.FaxNumber;

                    // add the fax file, in this case a simple txt file
                    outboundRequest.Files.Add(new FaxFile
                                                  {
                                                      FileType = FaxFileType.txt,
                                                      FileContents = Encoding.ASCII.GetBytes(testFax.FaxMessage)
                                                  });

                    // set a TransmissionID
                    outboundRequest.TransmissionControl.TransmissionID =
                        string.Format("WebTest-{0}", Guid.NewGuid()).Substring(0, 15);

                    outboundRequest.TransmissionControl.CustomerID = "MVC sample app";

                    // set the disposition settings
                    outboundRequest.DispositionControl.DispositionMethod = DispositionMethod.Email;
                    outboundRequest.DispositionControl.DispositionEmails.Add(new DispositionEmail
                                                                                 {
                                                                                     DispositionAddress =
                                                                                         testFax.DispositionEmail
                                                                                 });

                    // get an instance of OutboundClient
                    OutboundClient outboundClient = GetOutboundClient();

                    // send the OutboundRequest to efaxDeveloper
                    OutboundResponse outboundResponse = outboundClient.SendOutboundRequest(outboundRequest);

                    // check the status of the OutboundRequest
                    if (outboundResponse.StatusCode == StatusCode.Success)
                        ViewData["DOCID"] = outboundResponse.DOCID;
                    else
                        TempData["Error"] = outboundResponse.ErrorMessage;

                    return View();
                }
            }
            catch(Exception ex)
            {
                TempData["Error"] = ex.Message;
            }

            return View();

        }
       
        /// <summary>
        /// Get the status of a fax
        /// </summary>
        /// <param name="docId">docId of the fax</param>
        /// <returns></returns>
        public ActionResult FaxStatus(string docId)
        {
            // create an instance of OutboundRequest
            OutboundClient outboundClient = GetOutboundClient();

            // get the status of the outbound fax transmission for the given docId
            OutboundStatusResponse status = outboundClient.SendOutboundStatusRequest(null, docId);

            return View(status);
        }

        /// <summary>
        /// Check the inbound fax that were posted to ~/services/InboundFaxService.svc/receive
        /// </summary>
        /// <returns></returns>
        public ActionResult CheckInbound()
        {
            return View();
        }

        /// <summary>
        /// Get inbound fax info
        /// </summary>
        /// <param name="index">index of fax</param>
        /// <returns></returns>
        public ActionResult GetInbound(int? index)
        {
            // set default value of index to 0
            index = index ?? 0;

            // get the location of the inbound xml post by efaxDeveloper
            string inboundPath = WebConfigurationManager.AppSettings["InboundFax"];

            if (!string.IsNullOrEmpty(inboundPath) && Directory.Exists(inboundPath))
            {
                // get the inbound file based on the index requested
                string file = Directory.GetFiles(inboundPath).Skip(index.Value).FirstOrDefault();

                if(!string.IsNullOrEmpty(file))
                {
                    try
                    {
                        // create an instance of InboundClient
                        InboundClient inboundClient = new InboundClient();

                        using(FileStream fs = new FileStream(file, FileMode.Open))
                        {
                            // parse the xml with InboundClient
                            InboundPostRequest inboundPostRequest = inboundClient.DeserializeInboundPostRequest(fs);

                            // return some inbound fax information
                            return Json(new
                            {
                                id = Path.GetFileNameWithoutExtension(file),
                                Index = index,
                                inboundPostRequest.FaxControl.AccountID,
                                inboundPostRequest.FaxControl.PageCount,
                                inboundPostRequest.FaxControl.ANI,
                                inboundPostRequest.FaxControl.CSID,
                                DateReceived = inboundPostRequest.FaxControl.DateReceived.ToString("G"),
                                IsMultipage = inboundPostRequest.FaxControl.FileContents == null,
                            }, JsonRequestBehavior.AllowGet);
                        }
                    }
                    catch (Exception ex)
                    {
                        return Json(new
                        {
                            Index = index,
                            Error = ex.Message
                        }, JsonRequestBehavior.AllowGet);
                    }

                    
                }
            }

            return Json(null, JsonRequestBehavior.AllowGet);
        }

        /// <summary>
        /// Get the inbound fax document
        /// </summary>
        /// <param name="id">id (which is the xml filename without the extension)</param>
        /// <param name="page">page (if the document is nultipage)</param>
        /// <returns></returns>
        public ActionResult GetInboundImage(string id, int? page)
        {
            // get the location of the inbound xml post by efaxDeveloper
            string inboundPath = WebConfigurationManager.AppSettings["InboundFax"];

            // build the file name based on the id 
            string filePath = string.Format("{0}.xml", Path.Combine(inboundPath, id));

            // create an instance of InboundClient
            InboundClient inboundClient = new InboundClient();

            using (FileStream fs = new FileStream(filePath, FileMode.Open))
            {
                // parse the xml with InboundClient
                InboundPostRequest inboundPostRequest = inboundClient.DeserializeInboundPostRequest(fs);

                // check that the page requested exists
                if (page != null)
                    if (page.Value >= inboundPostRequest.FaxControl.PageCount)
                        return View("CheckInbound");

                string fileType;
                string fileName;

                // build the document file name
                if (page != null)
                    fileName = string.Format("{0}-{1}", inboundPostRequest.FaxControl.FaxName, page.Value);
                else
                    fileName = inboundPostRequest.FaxControl.FaxName;

                // add the document file name extension and content type
                if (inboundPostRequest.FaxControl.FileType == FaxFileType.tif)
                {
                    fileType = "image/tif";
                    fileName = string.Format("{0}.tif", fileName);
                }
                else
                {
                    fileType = "application/pdf";
                    fileName = string.Format("{0}.pdf", fileName);
                }

                // return document
                if (inboundPostRequest.FaxControl.FileContents != null)
                    return File(inboundPostRequest.FaxControl.FileContents, fileType, fileName);
                else if (page != null)
                    return File(inboundPostRequest.FaxControl.Pages[page.Value].PageContents, fileType, fileName);
            }

            return View("CheckInbound");
        }
    }
}
